/*
 * cbp.cc
 *
 *  Created on: Oct 15, 2015
 *    Author: Ali Rostami (WINLAB, Rutgers University)
 *		Email: rostami@winlab.rutgers.edu
 *		Homepage: alirostami.info
 */

#include "cbp.h"

namespace ns3 {

CBP::CBP()
:	smoothed_CBP_ (0),
	raw_CBP_ (0),
	cbp_calculation_window_ (100),
	idle_duration_ (0),
	idle_ratio_ (0),
	cbp_smooth_factor_ (0.5),
	start_time_ (Seconds(0)),
	first_CBP_calculation_ (true)
{
}

void
CBP::ScheduleCalculation()
{
	next_calculation_ = Simulator::Schedule (MilliSeconds (cbp_calculation_window_), &CBP::CalculateCBP, this);
}

void
CBP::CalculateCBP(void)
{
  double du;

  /*
   * Update the amount of the idle duration, then calculate the raw CBP and
   * smoothed CBP inside the CBP object m_CBP
   */

  if (phy_->IsStateIdle())  // check if current state is idle
		{
			if(start_time_ > phy_->GetLastIdleStartTime())
				{
					du = phy_->GetStateDuration().GetSeconds()*1000 - (start_time_ - phy_->GetLastIdleStartTime()).GetSeconds()*1000;
				}
			else
				{
					du = phy_->GetStateDuration().GetSeconds()*1000;
				}
		}
  else
		{
			du = 0;
		}

  idle_duration_ += du;

  // calculate raw CBP
  idle_ratio_ = idle_duration_ / (Simulator::Now ().GetSeconds() * 1000 - start_time_.GetSeconds() * 1000);
	double busyRatio = 1 - idle_ratio_;
	raw_CBP_ = busyRatio;

	if (first_CBP_calculation_)
		{
			smoothed_CBP_ = raw_CBP_;
			first_CBP_calculation_ = false;
		}
	else
		{
			smoothed_CBP_ = cbp_smooth_factor_*raw_CBP_ + (1-cbp_smooth_factor_)*smoothed_CBP_;
		}

	// Logging purpose
  Ptr<MultiApp> app= DynamicCast<MultiApp>(node_->GetApplication(1));
  std::cout<< "cbp " << GetRawCBP()
					 << " Id " << app->GetNodeID()
					 << " t " << Simulator::Now()
					 << std::endl;

  idle_duration_ = 0;
  start_time_ = Simulator::Now ();


  // Schedule next CBP calculation
  ScheduleCalculation ();
}

void
CBP::CBPStart()
{
	start_time_ = Simulator::Now();
  phy_ = DynamicCast<YansWifiPhy>(node_->GetDevice(0)->GetObject<WifiNetDevice>()->GetPhy());
  ScheduleCalculation ();
}

} //namespace ns3


