/*
 * multi-app-packet-sink.h
 *
 *  Created on: Oct 15, 2015
 *    Author: Ali Rostami (WINLAB, Rutgers University)
 *    Email: rostami@winlab.rutgers.edu
 *    Homepage: alirostami.info
 */

#ifndef SRC_APPLICATIONS_MODEL_MULTI_APP_PACKET_SINK_H_
#define SRC_APPLICATIONS_MODEL_MULTI_APP_PACKET_SINK_H_

#include "ns3/application.h"
#include "ns3/event-id.h"
#include "ns3/ptr.h"
#include "ns3/traced-callback.h"
#include "ns3/address.h"
#include "ns3/core-module.h"
#include "ns3/mobility-module.h"
#include <ns3/multiapp-packet.h>
#include "ns3/applications-module.h"
#include "ns3/simulator.h"
#include "multi-app.h"
#include <stdlib.h>

namespace ns3 {

class Address;
class Socket;
class Packet;

class MultiAppPacketSink : public Application
{
public:
  static TypeId GetTypeId (void);
  MultiAppPacketSink ();
  virtual ~MultiAppPacketSink ();

  /**
   * \return the total bytes received in this sink app
   */
  uint32_t GetTotalRx () const;
  inline uint32_t GetNodeID () {return nodeID; }

  /**
   * \return pointer to listening socket
   */
  Ptr<Socket> GetListeningSocket (void) const;

  /**
   * \return list of pointers to accepted sockets
   */
  std::list<Ptr<Socket> > GetAcceptedSockets (void) const;

  FILE*		trcFile;
//  bool		isLogger;
  double  rssi_;
  int sight_case[5000];

  Ptr<MobilityModel> mobilityModel;

protected:
  virtual void DoDispose (void);
  Address         m_local;        // Local address to bind to
   uint32_t        m_totalRx;      // Total bytes received
   TypeId          m_tid;          // Protocol TypeId

private:
  // inherited from Application base class.
  virtual void StartApplication (void);    // Called at time specified by Start
  virtual void StopApplication (void);     // Called at time specified by Stop

  void HandleRead (Ptr<Socket>);
  void HandleAccept (Ptr<Socket>, const Address& from);
  void HandlePeerClose (Ptr<Socket>);
  void HandlePeerError (Ptr<Socket>);
  void receiveLog(Ptr<Packet> packet,Ptr<Socket> socket);

  int nodeID;
  Ptr<Node> node;
  Ptr<Socket>     m_socket;       // Listening socket
  std::list<Ptr<Socket> > m_socketList; //the accepted sockets (TCP case)

  int slidingWindowSize;
  public:

  double localCBP;
  double RSSI;
  double PLCP;

  //*************************************************
  TracedCallback<Ptr<const Packet>, const Address &> m_rxTrace;

  EventId         RXMEvent;
  double          RXMInterval;
};

void RxDrop (Ptr<Node>, Ptr<const Packet>);
uint32_t ConvertMacAddressToId (Mac48Address);
void RssiTrace (Ptr<Node>, double);

} // namespace ns3

#endif /* SRC_APPLICATIONS_MODEL_MULTI_APP_PACKET_SINK_H_ */
