/*
 * multi-app.h
 *
 *  Created on: Oct 15, 2015
 *    Author: Ali Rostami (WINLAB, Rutgers University)
 *    Email: rostami@winlab.rutgers.edu
 *    Homepage: alirostami.info
 */

#ifndef SRC_APPLICATIONS_MODEL_MULTI_APP_H_
#define SRC_APPLICATIONS_MODEL_MULTI_APP_H_

#include <ns3/application.h>
#include <ns3/event-id.h>
#include <ns3/timer.h>
#include <ns3/qos-tag.h>
#include <ns3/wifi-module.h>
#include <ns3/data-rate.h>
#include <ns3/ptr.h>
#include "ns3/traced-callback.h"
#include "ns3/address.h"
#include <ns3/socket.h>
#include <ns3/cbp.h>
#include <ns3/multiapp-packet.h>
#include "multi-app-packet-sink.h"
#include <ns3/Rutgers-polygon.h>


namespace ns3{

class CBP;

enum ApplicationType {Baseline = 1, OnStreetPed = 2, MovingPed = 3, MulTxRates = 4};

class MultiApp : public Application
{
  friend class PolygonList;

public:

  // General functions
  static TypeId GetTypeId (void);
  MultiApp ();
  virtual ~ MultiApp ();
  Ptr<Socket> GetSocket (void) const;
  void SetMaxBytes (uint32_t maxBytes);

  // Indicate type of Congestion Control Algorithm
  void setAppType (void);

  Ptr<NormalRandomVariable> normRnd_;
  Ptr<UniformRandomVariable> unifRnd_;

  void startBaseline(void);	   // initialize Baseline algorithm
  void startStreetBaseline(void);  // initialize streetBaseline algorithm
  void startPedMoveBaseline(void); // initialize startPedMoveBaseline algorithm
  void startLIMERIC(void);	   // initialize LIMERIC algorithm
  void startDifferentFreq(void);

  void nextCBPCalSchedule(void);	// Schedule next CBP calculation and log
  void CBPCalculator(void);		// Calculates CBP over last CBP period
  void nextTxSchedule(void);
  void firstSchedule(void);

  Ptr<UniformRandomVariable> jitter;
  Ptr<UniformRandomVariable> randStart;

  inline void SetNodeType(uint32_t t) {node_type = t;}

  inline uint32_t GetNodeID () {return nodeID_; }
  inline uint32_t GetNodeType () {return node_type; }

  CBP* CBP_;
  FILE*		trcFile;
  std::string   sidewalkFilePath_;

//  bool		isLogger;
  uint32_t node_type;

private:

  virtual void StartApplication (void);    // Called at time specified by Start
  virtual void StopApplication (void);     // Called at time specified by Stop
  uint32_t	app_maxBytes;     // Limit total number of bytes sent
  Ptr<Socket>	app_socket;       // Associated socket
  Address	app_peer;         // Peer address
  bool		app_connected;    // True if connected
  TypeId	m_tid;


  DataRate	dataRate_;
  uint32_t	packetSize;
  uint32_t	msgRate_;
  uint32_t	cbpCalPeriod;
  QosTag	qosTag;
  uint32_t	msgSeq;		// message sequence number

  Ptr<Node>	node;
  Ptr<WifiNetDevice> nd;
  Ptr<MobilityModel> mobilityModel;

  uint32_t		nodeID_;
  double	recentMeasuredCBP;

  EventId	nextCBPCalEvent;
  EventId	nextTxEvent;
  EventId appStart;

  uint32_t nn_;
  uint32_t number_of_monitor_nodes_;
  uint32_t number_of_vehicles;

  double max_randomization;

  // Baseline = 1, OnStreetPed = 2, MovingPed = 3, MulTxRates = 4
  uint32_t	appType_;
  // 0 = only vehicles send packets, 1 = only pedestrians send packets, 2 = every nodes send
  uint32_t	sim_type;

  void sendPacket(void);
  PayloadMultiApp CreatePacket();

  PolygonList *sidewalk;

  // MovingPed
  double last_movement;
  double max_idle;
  double last_x;
  double last_y;
  double movement_threshold;
  void movementCheckTimer(void);
  void movementChecker(void);
  EventId next_check;

  // MulTxRates
  uint32_t  msgRate2_;


};

//CallBackFunction Handler
void idleStateTraceCBP (Ptr<Node> node, Time start, Time duration, enum WifiPhy::State state);

} // namespace ns3

#endif /* SRC_APPLICATIONS_MODEL_MULTI_APP_H_ */
